local addonName = ...
local core = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceEvent-3.0", "AceConsole-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)

Tracker = core

core.factions = {}
core.gainsReputation = {}
core.timerReputation = false

core.currencies = nil
core.gainsCurrency = {}
core.timerCurrency = false

--core.defaultColors = FACTION_BAR_COLORS
core.prettyColors = {
	[1] = { r = 0.63, g = 0.00, b = 0.00 }, -- Hated
	[2] = { r = 0.63, g = 0.00, b = 0.00 }, -- Hostile
	[3] = { r = 0.63, g = 0.00, b = 0.00 }, -- Unfriendly
	[4] = { r = 0.82, g = 0.67, b = 0.00 }, -- Neutral
	[5] = { r = 0.50, g = 0.74, b = 0.43 }, -- Friendly
	[6] = { r = 0.49, g = 0.73, b = 0.42 }, -- Honored
	[7] = { r = 0.47, g = 0.68, b = 0.47 }, -- Revered
	[8] = { r = 0.40, g = 0.62, b = 0.40 }, -- Exalted
}

core.paragonColor = { r = 0.26, g = 0.42, b = 1 }

core.defaults = {
	profile = {
		delay = 0.5,
		chat = DEFAULT_CHAT_FRAME:GetName(),
		reputationReplace = true,
		colors = true,
		changeReputationBar = true,
		changeParagonBar = true,
		currencyReplace = true,
	}
}


--local factionsList = TrackerFactionsList
local currenciesList = TrackerCurrenciesList

function core:OnInitialize()
	core.db = LibStub("AceDB-3.0"):New("TrackerDB", core.defaults, true)

	core:RegisterEvent("PLAYER_ENTERING_WORLD", "WorldEnter")
	core:RegisterEvent("ADDON_LOADED", "Loaded")

	core:RegisterEvent("CURRENCY_DISPLAY_UPDATE", "CurrenciesPreChanges")

	ChatFrame_AddMessageEventFilter("CHAT_MSG_COMBAT_FACTION_CHANGE", _ChatFilter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CURRENCY", _ChatFilter)

	hooksecurefunc("ReputationFrame_Update", core.ReputationFrame_Update)
end

function core:Loaded()
	core:UnregisterEvent("ADDON_LOADED")
	--core:updateColors()
end

function core:WorldEnter()
	core:UnregisterEvent("PLAYER_ENTERING_WORLD")
	core.factions = core:GetFactions()
	core.currencies = core:GetCurrencies()
end

--function core:updateColors()
--	if core.db.profile.colors then
--		FACTION_BAR_COLORS = core.prettyColors
--	else
--		FACTION_BAR_COLORS = core.defaultColors
--	end
--end

function core:ReputationFrame_Update(...)
	local numFactions = GetNumFactions()
	local factionOffset = FauxScrollFrame_GetOffset(ReputationListScrollFrame)

	for i = 1, NUM_FACTIONS_DISPLAYED do
		local factionIndex = factionOffset + i
		local factionRow = _G["ReputationBar" .. i]
		local factionBar = _G["ReputationBar" .. i .. "ReputationBar"]
		local factionStanding = _G["ReputationBar" .. i .. "ReputationBarFactionStanding"]

		if factionIndex <= numFactions then
			local _, _, standingID, _, _, _, _, _, _, _, _, _, _, factionID = GetFactionInfo(factionIndex)

			if factionID then
				if standingID == MAX_REPUTATION_REACTION and C_Reputation.IsFactionParagon(factionID) and core.db.profile.changeParagonBar then
					local currentValue, threshold, _, hasRewardPending = C_Reputation.GetFactionParagonInfo(factionID)
					local value = mod(currentValue, threshold)
					if hasRewardPending then
						value = value + threshold
					end
					factionRow.rolloverText = HIGHLIGHT_FONT_COLOR_CODE .. " " .. format(REPUTATION_PROGRESS_FORMAT, value, threshold) .. FONT_COLOR_CODE_CLOSE
					factionStanding:SetText(L["Paragon"])
					factionRow.standingText = L["Paragon"]

					factionBar:SetMinMaxValues(0, threshold)
					factionBar:SetValue(value)
					factionBar:SetStatusBarColor(core.paragonColor.r, core.paragonColor.g, core.paragonColor.b)

				else
					if core.db.profile.colors then
						local color = core.prettyColors[standingID]
						factionBar:SetStatusBarColor(color.r, color.g, color.b)
					end
				end

			end
		end
	end
end

function _ChatFilter(self, event, msg, ...)
	local skip = false
	if event == "CHAT_MSG_COMBAT_FACTION_CHANGE" then
		if string.match(msg, "%d") == nil then
			-- matched paragon reputation
			skip = false
		else
			if core.db.profile.delay > 0 then
				if core.timerReputation ~= nil then
					core:CancelTimer(core.timerReputation)
				end
				core.timerReputation = core:ScheduleTimer("FactionsChanges", core.db.profile.delay)
			else
				core:FactionsChanges()
			end
			skip = core.db.profile.reputationReplace
		end
	elseif event == "CHAT_MSG_CURRENCY" then
		skip = core.db.profile.currencyReplace
	end
	return skip, msg, ...
end

function core:CurrenciesPreChanges()
	if core.db.profile.delay > 0 then
		if core.timerCurrency ~= nil then
			core:CancelTimer(core.timerCurrency)
		end
		core.timerCurrency = core:ScheduleTimer("CurrenciesChanges", core.db.profile.delay)
	else
		core:CurrenciesChanges()
	end
end

function core:FactionsChanges()
	local factions = core.GetFactions()

	for i, new in pairs(factions) do
		if core.gainsReputation[i] == nil then
			core.gainsReputation[i] = 0
		end
		if core.factions[i] ~= nil then
			if not core.factions[i].paragon then
				local old = core.factions[i]
				local diff = new.barValue - old.barValue
				if diff ~= 0 then
					core.gainsReputation[i] = core.gainsReputation[i] + diff
					core:PrintReputation(new, diff, false)
				end
			else
				local old = core.factions[i]
				local diff = new.paragonValue - old.paragonValue
				if diff ~= 0 then
					core.gainsReputation[i] = core.gainsReputation[i] + diff
					core:PrintReputation(new, diff, false)
				end
			end
		else
			core.gainsReputation[i] = new.barValue
			core:PrintReputation(new, new.barValue, true)
		end
	end
	core.factions = factions
end

function core:CurrenciesChanges()
	if core.currencies == nil then
		return
	end

	local currencies = core.GetCurrencies()

	for i, new in pairs(currencies) do
		if core.gainsCurrency[i] == nil then
			core.gainsCurrency[i] = 0
		end
		if core.currencies[i] ~= nil then
			local old = core.currencies[i]
			local diff = new.currentAmount - old.currentAmount
			if diff ~= 0 then
				core.gainsCurrency[i] = core.gainsCurrency[i] + diff
				core:PrintCurrency(new, diff, false)
			end
		else
			core.gainsCurrency[i] = new.currentAmount
			core:PrintCurrency(new, new.currentAmount, true)
		end
	end
	core.currencies = currencies
end

function core:Test(faction_index, gains)
	if not core.factions[faction_index] then
		print("nops")
		return
	end
	local faction = core.factions[faction_index]
	core:debug(core.factions)
	core:PrintReputation(faction, gains, false)

end

function core:PrintReputation(new, diff, newFaction)
	local color = {
		main = "|cFF8080ff",
		white = "|cFFffffff",
		grey = "|cFF666666",
		off = "|cFFA7A7ff",
		points = "",
		gain = "|cFF40ff40",
		loss = "|cFFff4040"
	}

	local total = core.gainsReputation[new.factionID] or 0

	if diff == 0 then
		return
	elseif diff > 0 then
		color.points = color.gain
	else
		color.points = color.loss
	end

	--local currentIntervalName = new.intervalName
	local currentIntervalColor = core:RGBPercToHex(core.prettyColors[new.intervalId])
	local nextIntervalName = new.intervalNextName
	local nextIntervalColor = core:RGBPercToHex(core.prettyColors[new.intervalNextId])

	local maxBars = 20

	local currentPercent = 100 * new.current / new.currentMax
	--local nextPercent = 100 * new.currentNext / new.currentMax
	local diffPercent = 100 * diff / new.currentMax

	if new.paragon then
		diffPercent = 100 * diff / 10000
	end

	local str = color.main .. new.name .. ": " .. color.points .. diff .. " p." .. color.off .. " (" .. ("%.2f"):format(diffPercent) .. "%)"

	--if not newFaction then

	if new.paragon then
		local paragonValue = new.paragonValue % 10000
		--local paragonNext = new.paragonNext - paragonValue
		nextIntervalName = L["Paragon"]
		nextIntervalColor = core:RGBPercToHex(core.paragonColor)
		currentIntervalColor = nextIntervalColor
		currentPercent = 100 * paragonValue / 10000
	end

	local currentBars = maxBars * currentPercent / 100
	local bars = ""
	for i = 1, maxBars do
		if i <= currentBars then
			bars = bars .. currentIntervalColor .. "||" .. color.white
		else
			bars = bars .. color.grey .. "||" .. color.white
		end
	end

	str = str .. color.white .. " - " .. ("%.2f"):format(currentPercent) .. "% " .. L["to"] .. " " .. nextIntervalColor .. nextIntervalName .. color.white .. " [" .. bars .. "]"
	if total > 0 and not newFaction then
		str = str .. color.white .. " - " .. L["session"] .. ": " .. total
	end
	--end

	if core.db.profile.reputationReplace then
		core:_Print(str)
	end

	if core.db.profile.changeReputationBar then
		if not (new.watched or IsFactionInactive(new.index)) then
			SetWatchedFactionIndex(new.index)
		end
	end
end

function core:RGBPercToHex(color)
	return ("|cFF%02x%02x%02x"):format(color.r * 255, color.g * 255, color.b * 255)
end

function core:PrintCurrency(new, diff, newCurrency)
	local color = {
		main = "|cFF00aa00",
		off = "|cFF42cf42",
		points = "",
		gain = "|cFF40ff40",
		loss = "|cFFff4040"
	}

	local total = core.gainsCurrency[new.index]

	if diff == 0 then
		return
	elseif diff > 0 then
		color.points = color.main
	else
		color.points = color.loss
		diff = 0 - diff
	end

	local str = new.link .. color.main .. ": " .. color.points .. "x" .. diff .. "" .. color.main .. ". "

	str = str .. color.off .. "("

	str = str .. L["session"] .. ": " .. total .. ", "

	str = str .. L["total"] .. ": " .. new.currentAmount
	if new.totalMax > 0 then
		str = str .. "/" .. new.totalMax
	end

	str = str .. ")"

	--	if not newCurrency then
	--		str = str .. color.off .. "  (session: " .. total .. ", left: " .. new.currentNext .. " " .. new.intervalNextName .. ")"
	--	end

	if core.db.profile.currencyReplace then
		core:_Print(str)
	end
end

function core:GetFactions()
	local factions = {};

	ExpandAllFactionHeaders();

	for i = 1, GetNumFactions() do
		local
		name,
		description,
		standingId,
		barMin,
		barMax,
		barValue,
		atWarWith,
		canToggleAtWar,
		isHeader,
		isCollapsed,
		hasRep,
		isWatched,
		isChild,
		factionID,
		hasBonusRepGain,
		canBeLFGBonus = GetFactionInfo(i)

		if not isHeader and factionID ~= nil then

			if barValue == nil then
				barValue = 0;
			end

			local standingNextId = standingId + 1
			if standingNextId > MAX_REPUTATION_REACTION then
				standingNextId = MAX_REPUTATION_REACTION
			end
			local standingName = _G["FACTION_STANDING_LABEL" .. standingId];
			local standingNextName = _G["FACTION_STANDING_LABEL" .. standingNextId];

			local currentAmount = barValue - barMin;
			local currentMax = barMax - barMin;

			local paragon, paragonValue, paragonNext, paragonQuest, paragonReward = C_Reputation.IsFactionParagon(factionID), 0, 0, nil, false
			if paragon then
				paragonValue, paragonNext, paragonQuest, paragonReward = C_Reputation.GetFactionParagonInfo(factionID);

				if paragonReward then
					core:_Print("|cFF8080ff" .. name .. "|r has a paragon reward!")
				end
			end

			local faction = {
				factionID = factionID,
				index = i,
				name = name,
				watched = isWatched,
				intervalId = standingId,
				intervalName = standingName,
				intervalNextId = standingNextId,
				intervalNextName = standingNextName,
				current = currentAmount,
				currentMax = currentMax,
				currentNext = currentMax - currentAmount,
				barMin = barMin,
				barMax = barMax,
				barValue = barValue,
				paragon = paragon,
				paragonValue = paragonValue,
				paragonNext = paragonNext,
				paragonQuest = paragonQuest,
				paragonReward = paragonReward,
			}

			factions[factionID] = faction
		end
	end

	return factions
end

function core:GetCurrencies()
	local currencies = {};

	for _, id in pairs(currenciesList) do
		local name, currentAmount, texture, earnedThisWeek, weeklyMax, totalMax, isDiscovered, rarity = GetCurrencyInfo(id);

		if name ~= nil and isDiscovered then

			local currency = {
				index = id,
				name = name,
				texture = texture,
				link = GetCurrencyLink(id, currentAmount),
				rarity = rarity,
				currentAmount = currentAmount,
				weekAmount = earnedThisWeek,
				weeklyMax = weeklyMax,
				totalMax = totalMax,
				isDiscovered = isDiscovered
			}

			currencies[id] = currency
		end
	end

	return currencies;
end

function core:_Print(...)
	local chat = getglobal(core.db.profile.chat)
	if chat == nil then
		chat = DEFAULT_CHAT_FRAME
	end
	chat:AddMessage(...)
end

function core:debug(tData)
	if ViragDevTool_AddData then
		ViragDevTool_AddData(tData, "debug")
	end
end
